"""Stub file for the '_json' module."""
# This is an autogenerated file. It serves as a starting point
# for a more precise manual annotation of this module.
# Feel free to edit the source below, but remove this header when you do.

from typing import Any, List, Tuple, Dict, Generic

def encode_basestring_ascii(*args, **kwargs) -> str:
    raise TypeError()

def scanstring(a, b, *args, **kwargs) -> tuple:
    raise TypeError()


class Encoder(object): ...

class Scanner(object): ...
